CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
83 lines
3.5 KiB
C++
83 lines
3.5 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
#pragma once
|
|
|
|
#include "math_common.hh"
|
|
#include "validators.hh"
|
|
|
|
namespace cg = cooperative_groups;
|
|
|
|
#define CAST_HALF2_KERNEL_DEF(func_name, T) \
|
|
__global__ void func_name##_kernel(T* const ys, const size_t num_xs, Float16* const xs) { \
|
|
const auto tid = cg::this_grid().thread_rank(); \
|
|
const auto stride = cg::this_grid().size(); \
|
|
\
|
|
for (size_t i = tid; i < num_xs; i += stride) { \
|
|
ys[i] = func_name(__half2{xs[i], -xs[i]}); \
|
|
} \
|
|
}
|
|
|
|
#define CAST_BINARY_HALF2_KERNEL_DEF(func_name, T) \
|
|
__global__ void func_name##_kernel(T* const ys, const size_t num_xs, Float16* const x1s, \
|
|
Float16* const x2s) { \
|
|
const auto tid = cg::this_grid().thread_rank(); \
|
|
const auto stride = cg::this_grid().size(); \
|
|
\
|
|
for (size_t i = tid; i < num_xs; i += stride) { \
|
|
ys[i] = func_name(__half2{x1s[i], -x1s[i]}, __half2{x2s[i], -x2s[i]}); \
|
|
} \
|
|
}
|
|
|
|
template <typename VB> class Float2Validator : public MatcherBase<float2> {
|
|
public:
|
|
Float2Validator(const float2& target, const VB& vb)
|
|
: first_matcher_{vb(target.x)}, second_matcher_{vb(target.y)} {}
|
|
|
|
bool match(const float2& val) const override {
|
|
return first_matcher_->match(val.x) && second_matcher_->match(val.y);
|
|
}
|
|
|
|
std::string describe() const override {
|
|
return "<" + first_matcher_->describe() + ", " + second_matcher_->describe() + ">";
|
|
}
|
|
|
|
private:
|
|
decltype(std::declval<VB>()(float())) first_matcher_;
|
|
decltype(std::declval<VB>()(float())) second_matcher_;
|
|
};
|
|
|
|
template <typename ValidatorBuilder>
|
|
auto Float2ValidatorBuilderFactory(const ValidatorBuilder& vb) {
|
|
return [=](const float2& t, auto&&...) {
|
|
return std::make_unique<Float2Validator<ValidatorBuilder>>(t, vb);
|
|
};
|
|
}
|
|
|
|
template <typename VB> class Half2Validator : public MatcherBase<__half2> {
|
|
public:
|
|
Half2Validator(const __half2& target, const VB& vb)
|
|
: first_matcher_{vb(target.data.x)}, second_matcher_{vb(target.data.y)} {}
|
|
|
|
bool match(const __half2& val) const override {
|
|
return first_matcher_->match(val.data.x) && second_matcher_->match(val.data.y);
|
|
}
|
|
|
|
std::string describe() const override {
|
|
return "<" + first_matcher_->describe() + ", " + second_matcher_->describe() + ">";
|
|
}
|
|
|
|
private:
|
|
decltype(std::declval<VB>()(Float16())) first_matcher_;
|
|
decltype(std::declval<VB>()(Float16())) second_matcher_;
|
|
};
|
|
|
|
template <typename ValidatorBuilder> auto Half2ValidatorBuilderFactory(const ValidatorBuilder& vb) {
|
|
return [=](const __half2& t, auto&&...) {
|
|
return std::make_unique<Half2Validator<ValidatorBuilder>>(t, vb);
|
|
};
|
|
}
|